﻿#include "jsb_dotShare_manual.h"

#include "C2DXShareSDK.h"
#include "ScriptingCore.h"
#include "js_manual_conversions.h"
#include <string>

using namespace std;
using namespace cocos2d;
using namespace cn::sharesdk;

#define DOTSHARE_NULL_THEN_RETURN(condition) do { \
	if( ! (condition) ) return; \
	} while(0)

#ifdef COCOS2D_35X
void dotshare_get_or_create_js_obj(JSContext* cx, JS::HandleObject obj, const std::string &name, JS::MutableHandleObject jsObj)
{

	JS::RootedValue nsval(cx);
	JS_GetProperty(cx, obj, name.c_str(), &nsval);
	if (nsval == JSVAL_VOID) {
		jsObj.set(JS_NewObject(cx, NULL, JS::NullPtr(), JS::NullPtr()));
		nsval = OBJECT_TO_JSVAL(jsObj);
		JS_SetProperty(cx, obj, name.c_str(), nsval);
	} else {
		jsObj.set(nsval.toObjectOrNull());
	}
}

#else

void dotshare_get_or_create_js_obj(JSContext* cx, JSObject* obj, const std::string &name, JSObject** jsObj)
{
#ifdef COCOS2D_3X
	JS::RootedValue nsval(cx);
	JS::RootedObject ns(cx);
#else
	jsval nsval;
	JSObject* ns;
#endif
	JS_GetProperty(cx, obj, name.c_str(), &nsval);
	if(nsval == JSVAL_VOID)
	{
		ns = JS_NewObject(cx, NULL, NULL, NULL);
		nsval = OBJECT_TO_JSVAL(ns);
#ifdef COCOS2D_3X
		JS_SetProperty(cx, obj, name.c_str(), nsval);
#else
		JS_SetProperty(cx, obj, name.c_str(), &nsval);
#endif
	}
	else
	{
		JS_ValueToObject(cx, nsval, &ns);
	}
	*jsObj = ns;
}
#endif

void dotshare_send_data_to_js(JSContext *cx, jsval* vp, jsval data, int argc)
{
#ifdef COCOS2D_35X
	JS::CallArgs argv = JS::CallArgsFromVp(argc, vp);
	argv.rval().set(data);
#else
	JS_SET_RVAL(cx, vp, data);
#endif
}

void dotshare_get_string_from_jsval(JSContext *cx, jsval* vp, int argc, int index, const char** ret)
{
	if(index >= argc)
	{
		return;
	}
	JSString *tmp = NULL;
#ifdef COCOS2D_35X
	JS::CallArgs argv = JS::CallArgsFromVp(argc, vp);
	JS::HandleValue v = argv.get(index);
	if(v.isString() || v.isNumber())
	{
		tmp = JS::ToString(cx, v);
	}
#else
	jsval* argv = JS_ARGV(cx, vp);
#ifdef COCOS2D_2X
	tmp = JS_ValueToString(cx, argv[index]);
#else 
	tmp = JS::ToString(cx, JS::RootedValue(cx,argv[index]));
#endif
#endif
	DOTSHARE_NULL_THEN_RETURN(tmp);
#ifdef COCOS2D_2X
	unsigned short* pStrUTF16 = (unsigned short*)JS_GetStringCharsZ(cx, tmp);
	*ret = cc_utf16_to_utf8(pStrUTF16, -1, NULL, NULL);
#else
	*ret = JS_EncodeString(cx, tmp);
#endif
}


void dotshare_get_int_from_jsval(JSContext *cx, jsval* vp, int argc, int index, int* ret)
{
	if(index >= argc)
	{
		return;
	}
#ifdef COCOS2D_35X
	JS::CallArgs argv = JS::CallArgsFromVp(argc, vp);
	JS::ToInt32(cx,argv.get(index), ret);
#else
	jsval* argv = JS_ARGV(cx, vp);
#ifdef COCOS2D_2X
	JS::ToInt32(cx,argv[index], ret);
#elif defined COCOS2D_3X
	JS::ToInt32(cx,JS::RootedValue(cx,argv[index]), ret);
#endif
#endif
}

void shareResultHandler(C2DXResponseState state, C2DXPlatType platType, CCDictionary *shareInfo, CCDictionary *error)
{
 	std::string fun;
	
	switch (state) {
	case C2DXResponseStateSuccess:		
		fun = "DotShare.callback(1);";
		break;
	case C2DXResponseStateFail:		
		fun = "DotShare.callback(2);";
		break;
	case C2DXResponseStateCancel:		
		fun = "DotShare.callback(3);";
		break;
	default:
		break;
	}
	jsval ret; 	
	ScriptingCore::getInstance()->evalString(fun.c_str(), &ret); 
}

//分享.
JSBool js_DOTSHARE_share(JSContext* cx, uint32_t argc, jsval* vp)
{
	//分享平台
	if(argc == 1)
	{
		int platform;	
		dotshare_get_int_from_jsval(cx, vp, argc, 0, &platform);
		
		CCDictionary *content = CCDictionary::create();
		content -> setObject(CCString::create("送！送！送！"), "content");
		content -> setObject(CCString::create("http://img0.bdstatic.com/img/image/shouye/systsy-11927417755.jpg"), "image");
		content -> setObject(CCString::create("美食连萌"), "title");
		content -> setObject(CCString::create("测试描述"), "description");
		content -> setObject(CCString::create("http://7.dd1.app1101321802.twsapp.com/h5/"), "url");
		content -> setObject(CCString::createWithFormat("%d", C2DXContentTypeNews), "type");
		content -> setObject(CCString::create("http://baidu.com/"), "siteUrl");
		content -> setObject(CCString::create("ShareSDK"), "site");
		//content -> setObject(CCString::create("http://mp3.mwap8.com/destdir/Music/2009/20090601/ZuiXuanMinZuFeng20090601119.mp3"), "musicUrl");
		//content -> setObject(CCString::create("extInfo"), "extInfo");

		C2DXPlatType platType = C2DXShareSDK::gameIdxToPlatformId(platform);

		C2DXShareSDK::shareContent(platType, content, shareResultHandler);
	
		return DOTSHARE_TRUE;
	}
	return DOTSHARE_FALSE;
}

JSBool js_DOTSHARE_shareContent(JSContext* cx, uint32_t argc, jsval* vp)
{
cocos2d::log("ios share...");
  if(argc != 2) return DOTSHARE_FALSE;
  
  int platform;
	
	dotshare_get_int_from_jsval(cx, vp, argc, 0, &platform);
  C2DXPlatType platType = C2DXShareSDK::gameIdxToPlatformId(platform);
    
#if CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID
  const char* content;
	dotshare_get_string_from_jsval(cx, vp, argc, 1, &content);
	C2DXShareSDK::shareContent2(platType, content, shareResultHandler);
	JS_free(cx, (void*)content);	
	return DOTSHARE_TRUE;
  
#elif CC_TARGET_PLATFORM == CC_PLATFORM_IOS
cocos2d::log("ios in...");
  JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
  JS::RootedObject obj(cx, args.get(1).toObjectOrNull());
  if(!obj) return DOTSHARE_FALSE;

  CCDictionary *content = CCDictionary::create();
  JS::RootedValue val(cx);
  std::string str;
  if(JS_GetProperty(cx, obj, "content", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "content");
  if(JS_GetProperty(cx, obj, "image", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "image");
  if(JS_GetProperty(cx, obj, "title", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "title");
  if(JS_GetProperty(cx, obj, "description", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "description");
  if(JS_GetProperty(cx, obj, "url", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "url");
  if(JS_GetProperty(cx, obj, "type", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "type");
  if(JS_GetProperty(cx, obj, "siteUrl", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "siteUrl");
  if(JS_GetProperty(cx, obj, "site", &val) && !val.isNullOrUndefined() && jsval_to_std_string(cx, val, &str))
    content-> setObject(CCString::create(str.c_str()), "site");
cocos2d::log("ios in2...");
  C2DXShareSDK::shareContent(platType, content, shareResultHandler);
  return DOTSHARE_TRUE;
#endif
}


//客户端是否安装
JSBool js_DOTSHARE_isClientInstalled(JSContext* cx, uint32_t argc, jsval* vp)
{
	if(argc == 1)
	{
		int platform;		
		dotshare_get_int_from_jsval(cx, vp, argc, 0, &platform);
		C2DXPlatType platType = C2DXShareSDK::gameIdxToPlatformId(platform);
		bool value = C2DXShareSDK::isClientInstalled(platType);

		jsval jsret; 
		jsret = BOOLEAN_TO_JSVAL(value); 
		dotshare_send_data_to_js(cx, vp, jsret, argc); 

		return DOTSHARE_TRUE;
	}
	return DOTSHARE_FALSE;
}



void register_dotShare_function(JSContext* cx,  dotShareSObjectPtr obj)
{
#ifdef COCOS2D_35X
	JS::RootedObject ns(cx);
#else
	JSObject* ns;
#endif
	dotshare_get_or_create_js_obj(cx, obj, "DotShareSDK", &ns);
	JS_DefineFunction(cx, ns, "share", js_DOTSHARE_share, 1, JSPROP_READONLY|JSPROP_PERMANENT);
	JS_DefineFunction(cx, ns, "shareContext", js_DOTSHARE_shareContent, 1, JSPROP_READONLY|JSPROP_PERMANENT);
	JS_DefineFunction(cx, ns, "isClientInstalled", js_DOTSHARE_isClientInstalled, 1, JSPROP_READONLY|JSPROP_PERMANENT);
}
